
function love.load()
	
	width, height = 480, 320 -- units, not pixels
	unit = 20
	scale = 2
--	scale = 6
	love.window.setMode( width*scale, height*scale, {resizable=true} )
	love.graphics.setDefaultFilter( 'nearest', 'nearest' )
	love.graphics.setLineStyle( "rough" )

	map = {}
	for i = 0, (width-unit)/unit do
		map[i] = {}
		for j = 0, (height-unit)/unit do
			map[i][j] = 0
		end
	end
	
--	love.mouse.setGrabbed(true)
	mouse_line = {points = {}, pressed = false, last = {x=0, y=0}}
	lines = {}
	agents = {}
	
--	local line = {30, 30, 60, 80, 80, 50, 300, 10}
--	table.insert(lines, line)
--	create_agent (line)

	topology = {}
	crossings = {}
	
	crosslines = {}
	
--	get_intersection (ax, ay, bx, by, cx, cy, dx, dy)
--	get_intersection (10, 10, 300, 300, 200, 10 , 10, 200)
end


function create_agent (line)
	local agent = {}
	agent.line = line
	agent.t = 0
	agent.s = 0
	agent.max_v = 20
	agent.sign = 1
--	agent.position = {x=line[1], y=line[2]}
	agent.x = line[1]
	agent.y = line[2]
	agent.n = 1 -- number of section
	agent.rs = 0 -- section path relative to section length
	
	
	table.insert (agents, agent)
end

function love.update(dt)
	for i, agent in pairs (agents) do
		agent.s = agent.s + dt*agent.max_v
		agent.t = agent.t + dt
		local line = agent.line
		local n = agent.n
		local x1, y1 = line[n], line[n+1]
		local x2, y2 = line[n+2], line[n+3]
		
		local sq_section = (x2-x1)^2+(y2-y1)^2
		local sq_agent = (agent.x-x1)^2+(agent.y-y1)^2
		agent.rs = (sq_agent/sq_section)^0.5
		if agent.rs > 1 then
			n = n + 2
			if not line[n+2] then 
				n = 1
				agent.x, agent.y = line[n], line[n+1]
			end
			agent.n = n
			x1, y1 = line[n], line[n+1]
			x2, y2 = line[n+2], line[n+3]
			sq_section = (x2-x1)^2+(y2-y1)^2
			
			local s = sq_section^0.5
			agent.vx = (x2-x1)/s*agent.max_v
			agent.vy = (y2-y1)/s*agent.max_v
		elseif not agent.vx then
			local s = sq_section^0.5
			agent.vx = (x2-x1)/s*agent.max_v
			agent.vy = (y2-y1)/s*agent.max_v
		end

		agent.x = agent.x + dt*agent.vx
		agent.y = agent.y + dt*agent.vy
	end
end

function draw_mouse_line ()
	local line = {}
	for i, point in ipairs (mouse_line.points) do
		table.insert (line, point.x/scale)
		table.insert (line, point.y/scale)
	end
	if #line >4 then
		love.graphics.setColor(1,1,0)
		love.graphics.line(line)
	end
--	table.insert (lines, line)
end

function draw_map ()
	love.graphics.setColor(.1,.1,.1)
	for i, js in pairs (map) do
		for j, value in pairs (js) do
			love.graphics.rectangle('line', i*unit, j*unit, unit, unit)
		end
	end
end

function draw_mouse_poiner ()
	if mouse_line.pressed then
		love.graphics.circle('line', love.mouse.getX( )/scale, love.mouse.getY( )/scale, 0.5*unit)
	else
		love.graphics.circle('line', love.mouse.getX( )/scale, love.mouse.getY( )/scale, 0.25*unit)
	end
end


function draw_lines ()
	local line_width = love.graphics.getLineWidth( )
	love.graphics.setLineWidth( 20 )

	love.graphics.setColor(1,1,1)
	
	for i, line in pairs (lines) do
		love.graphics.circle('fill', line[1], line[2], 20/2)
		love.graphics.line(line)
		love.graphics.circle('fill', line[#line-1], line[#line], 20/2)
	end
	love.graphics.setLineWidth( line_width )

end


function draw_crossings ()
	love.graphics.setColor(0,1,0)
	for i, crossing in pairs (crossings) do
		love.graphics.circle ('line', crossing.x, crossing.y, 5)
--		love.graphics.print((crossing.x ..' '.. crossing.y), crossing.x, crossing.y)
	end
end


function draw_crosslines ()
	love.graphics.setColor(0,1,0)
	for i, line in pairs (crosslines) do
		love.graphics.line(line)
		
	end
end


function love.draw()
	
	love.graphics.scale(scale)
	
	draw_map ()

	draw_lines ()

	draw_mouse_line ()
	
	love.graphics.setColor(1,0,0)
	for i, agent in pairs (agents) do
		local s = (agent.n-1)/2+agent.rs
		local sl = 0.25
		love.graphics.circle('line', agent.x, agent.y, 0.25*unit)
		love.graphics.line(agent.x-sl*agent.vx, agent.y-sl*agent.vy, agent.x+sl*agent.vx, agent.y+sl*agent.vy)
--		love.graphics.print(string.format ("%#0.1f", s), agent.x, agent.y)
--		love.graphics.print(string.format ("%#0.2f", agent.t), agent.x, agent.y)
--		love.graphics.print(string.format ("%#0.2f", agent.s), agent.x, agent.y+15)
	end

	draw_crossings ()
	draw_crosslines ()
	
	draw_mouse_poiner ()
	
	
	love.graphics.print (tostring(mouse_line.pressed))
	love.graphics.print (tostring(#mouse_line.points), 0, 15)
	love.graphics.print (tostring(#lines), 0, 30)
end


function love.mousepressed(x, y, button, istouch)
	if button == 1 then
		mouse_line.pressed = true
		mouse_line.last = {x=x, y=y}
		mouse_line.points = {}
	end
end

function love.mousemoved( x, y, dx, dy, istouch )
	local ds = 0.5*unit/scale
	if mouse_line.pressed then
		if ((x-mouse_line.last.x)^2+(y-mouse_line.last.y)^2) >= (ds)^2 then
			local point = mouse_line.last
			mouse_line.last = {x=x, y=y}
			table.insert (mouse_line.points, point)
		end
	end
end

function smooth_line (line, n)
	n = n or 1
	
	local sline = {}
	sline[1] = line[1]
	sline[2] = line[2]
	sline[#line-1] = line[#line-1]
	sline[#line] = line[#line]
	for j = 1, n do
		for i = 3, #line-3, 2 do
			sline[i] = 0.5*line[i] + 0.25*line[i-2] + 0.25*line[i+2]
			sline[i+1] = 0.5*line[i+1] + 0.25*line[i-1] + 0.25*line[i+3]
		end
		line = sline
	end
	return line
end

function get_intersection (ax, ay, bx, by, cx, cy, dx, dy) -- start end start end
	local d = (ax-bx)*(cy-dy)-(ay-by)*(cx-dx)
	if d == 0 then return end
	local a, b = ax*by-ay*bx, cx*dy-cy*dx
	local x = (a*(cx-dx) - b*(ax-bx))/d
	local y = (a*(cy-dy) - b*(ay-by))/d
	if x <= math.max(ax, bx) and x >= math.min(ax, bx) and
		x <= math.max(cx, dx) and x >= math.min(cx, dx) then
			return {x=x, y=y}
	end
end

function find_crossings (line)
	for i, line2 in pairs (lines) do
		for j = 1, #line2-3, 2 do
			for k = 1, #line-3, 2 do
				local pos = get_intersection(
					line[k], line[k+1], line[k+2], line[k+3], 
					line2[j],line2[j+1],line2[j+2], line2[j+3])
				if pos then
					table.insert (crossings, pos)
				end
			end
		end
	end
end

function love.mousereleased(x, y, button)
	if button == 1 then
		mouse_line.pressed = false
		local line = {}
		for i, point in ipairs (mouse_line.points) do
			table.insert (line, point.x/scale)
			table.insert (line, point.y/scale)
		end
		if #line > 4 then
			line = smooth_line (line, 10)
		end
		if #line > 2 then
			find_crossings (line)
			
			table.insert (lines, line)
			create_agent (line)
		end
		mouse_line.points = {}
	end
end

function love.keypressed(key, scancode, isrepeat)
	if key == "escape" then
		love.event.quit()
	elseif key == "kp+" then
		scale = scale*2^(1/2)
	elseif key == "kp-" then
		scale = scale/2^(1/2)
	end
end